@@ -3,7 +3,7 @@ import math
 from typing import Optional, Type, TypeVar
 
 from . import constant
-from .vec import Vector, Quaterion
+from .vec import Vector, Quaternion
 from .__export import calls, funcs, vcmpEntityPool, vcmpPlayerOption, vcmpPlayerState, vcmpVehicleSync, vcmpVehicleOption
 from .streams import WriteStream
 
@@ -135,6 +135,14 @@ class Player:
     def controllable(self, controllable: bool):
         funcs.set_player_option(self._id, vcmpPlayerOption.vcmpPlayerOptionControllable, controllable)
 
+    @property
+    def frozen(self) -> bool:
+        return not self.controllable
+    
+    @frozen.setter
+    def frozen(self, frozen: bool):
+        self.controllable = not frozen
+
     @property
     def drive_by(self) -> bool:
         return funcs.get_player_option(self._id, vcmpPlayerOption.vcmpPlayerOptionDriveBy)
@@ -223,6 +231,14 @@ class Player:
     def secondary_world(self, secondary_world: int):
         funcs.set_player_secondary_world(self._id, secondary_world)
 
+    @property
+    def sec_world(self) -> int:
+        return self.secondary_world
+    
+    @sec_world.setter
+    def sec_world(self, secondary_world: int):
+        self.secondary_world = secondary_world
+
     @property
     def unique_world(self) -> int:
         return funcs.get_player_unique_world(self._id)
@@ -375,7 +391,11 @@ class Player:
         if vehicle is not None:
             id = vehicle.id
         if id is not None and funcs.check_entity_exists(vcmpEntityPool.vcmpEntityPoolVehicle, id):
-            funcs.put_player_in_vehicle(self._id, id, 0, 0, 1)
+            origin_id = funcs.get_player_vehicle_id(self._id)
+            if origin_id != id:
+                # remove from old vehicle
+                funcs.remove_player_from_vehicle(self._id)
+                funcs.put_player_in_vehicle(self._id, id, 0, 0, 1)
         else:
             funcs.remove_player_from_vehicle(self._id)
 
@@ -641,6 +661,30 @@ class Player:
         """
         funcs.kill_player(self._id)
 
+    def add_position(self, position: Vector):
+        """
+        Add position to the player position
+        """
+
+        new_pos = Vector(
+            self.position.x + position.x,
+            self.position.y + position.y,
+            self.position.z + position.z
+        )
+        self.position = new_pos
+
+    def add_speed(self, speed: Vector):
+        """
+        Add speed to the player speed
+        """
+
+        new_speed = Vector(
+            self.speed.x + speed.x,
+            self.speed.y + speed.y,
+            self.speed.z + speed.z
+        )
+        self.speed = new_speed
+
     def __repr__(self) -> str:
         return f"Player(id={self.id}, name='{self.name}')"
 
@@ -649,11 +693,14 @@ class Vehicle:
     def __init__(self, vehicle_id: int):
         self._id = vehicle_id
 
-
     @property
     def id(self):
         return self._id
 
+    @property
+    def is_alive(self):
+        return funcs.check_entity_exists(vcmpEntityPool.vcmpEntityPoolVehicle, self._id)
+
     @property
     def doors_locked(self):
         """
@@ -814,7 +861,9 @@ class Vehicle:
         """
         Get the vehicle position
         """
-        return funcs.get_vehicle_position(self._id)
+        return Vector(
+            **funcs.get_vehicle_position(self._id)
+        )
 
     @position.setter
     def position(self, value: Vector):
@@ -828,10 +877,10 @@ class Vehicle:
         """
         Get the vehicle rotation
         """
-        return Quaterion(**funcs.get_vehicle_rotation(self._id))
+        return Quaternion(**funcs.get_vehicle_rotation(self._id))
 
     @rotation.setter
-    def rotation(self, value: Quaterion):
+    def rotation(self, value: Quaternion):
         """
         Set the vehicle rotation
         """
@@ -856,7 +905,9 @@ class Vehicle:
         """
         Get the vehicle speed
         """
-        return Vector(**funcs.get_vehicle_speed(self._id, False))
+        return Vector(
+            **funcs.get_vehicle_speed(self._id, False)
+        )
     
     @speed.setter
     def speed(self, value: Vector):
@@ -870,7 +921,9 @@ class Vehicle:
         """
         Get the vehicle relative speed
         """
-        return Vector(**funcs.get_vehicle_speed(self._id, True))
+        return Vector(
+            **funcs.get_vehicle_speed(self._id, True)
+        )
     
     @relative_speed.setter
     def relative_speed(self, value: Vector):
@@ -884,7 +937,9 @@ class Vehicle:
         """
         Get the vehicle turn speed
         """
-        return Vector(**funcs.get_vehicle_turn_speed(self._id, False))
+        return Vector(
+            **funcs.get_vehicle_turn_speed(self._id, False)
+        )
 
     @turn_speed.setter
     def turn_speed(self, value: Vector):
@@ -898,7 +953,9 @@ class Vehicle:
         """
         Get the vehicle relative turn speed
         """
-        return Vector(**funcs.get_vehicle_turn_speed(self._id, True))
+        return Vector(
+            **funcs.get_vehicle_turn_speed(self._id, True)
+        )
 
     @relative_turn_speed.setter
     def relative_turn_speed(self, value: Vector):
@@ -912,7 +969,9 @@ class Vehicle:
         """
         Get the vehicle spawn position
         """
-        return Vector(**funcs.get_vehicle_spawn_position(self._id))
+        return Vector(
+            **funcs.get_vehicle_spawn_position(self._id)
+        )
 
     @spawn_position.setter
     def spawn_position(self, value: Vector):
@@ -926,10 +985,10 @@ class Vehicle:
         """
         Get the vehicle spawn rotation
         """
-        return Quaterion(**funcs.get_vehicle_spawn_rotation(self._id))
+        return Quaternion(**funcs.get_vehicle_spawn_rotation(self._id))
 
     @spawn_rotation.setter
-    def spawn_rotation(self, value: Quaterion):
+    def spawn_rotation(self, value: Quaternion):
         """
         Set the vehicle spawn rotation
         """
@@ -940,7 +999,9 @@ class Vehicle:
         """
         Get the vehicle spawn rotation euler
         """
-        return Vector(**funcs.get_vehicle_spawn_rotation_euler(self._id))
+        return Vector(
+            **funcs.get_vehicle_spawn_rotation_euler(self._id)
+        )
     
     @spawn_rotation_euler.setter
     def spawn_rotation_euler(self, value: Vector):
@@ -1178,6 +1239,121 @@ class Vehicle:
         """
         funcs.reset_inst_handling(self._id)
 
+    def add_position(self, position: Vector):
+        """
+        Add position to the vehicle position
+        """
+        new_pos = Vector(
+            self.position.x + position.x,
+            self.position.y + position.y,
+            self.position.z + position.z
+        )
+        self.position = new_pos
+
+    def add_speed(self, speed: Vector):
+        """
+        Add speed to the vehicle speed
+        """
+        new_speed = Vector(
+            self.speed.x + speed.x,
+            self.speed.y + speed.y,
+            self.speed.z + speed.z
+        )
+        self.speed = new_speed
+
+    def add_relative_speed(self, speed: Vector):
+        """
+        Add relative speed to the vehicle speed
+        """
+        new_speed = Vector(
+            self.relative_speed.x + speed.x,
+            self.relative_speed.y + speed.y,
+            self.relative_speed.z + speed.z
+        )
+        self.relative_speed = new_speed
+
+    def add_turn_speed(self, speed: Vector):
+        """
+        Add turn speed to the vehicle turn speed
+        """
+        new_speed = Vector(
+            self.turn_speed.x + speed.x,
+            self.turn_speed.y + speed.y,
+            self.turn_speed.z + speed.z
+        )
+        self.turn_speed = new_speed
+
+    def add_relative_turn_speed(self, speed: Vector):
+        """
+        Add relative turn speed to the vehicle turn speed
+        """
+        new_speed = Vector(
+            self.relative_turn_speed.x + speed.x,
+            self.relative_turn_speed.y + speed.y,
+            self.relative_turn_speed.z + speed.z
+        )
+        self.relative_turn_speed = new_speed
+
+    def add_rotation(self, rotation: Quaternion):
+        """
+        Add rotation to the vehicle rotation
+        """
+        new_rotation = Quaternion(
+            self.rotation.x + rotation.x,
+            self.rotation.y + rotation.y,
+            self.rotation.z + rotation.z,
+            self.rotation.w + rotation.w
+        )
+        self.rotation = new_rotation
+
+    def add_rotation_euler(self, rotation: Vector):
+        """
+        Add rotation to the vehicle rotation
+        """
+        new_rotation = Vector(
+            self.rotation_euler.x + rotation.x,
+            self.rotation_euler.y + rotation.y,
+            self.rotation_euler.z + rotation.z
+        )
+        self.rotation_euler = new_rotation
+
+    def add_spawn_position(self, position: Vector):
+        """
+        Add position to the vehicle spawn position
+        """
+        new_pos = Vector(
+            self.spawn_position.x + position.x,
+            self.spawn_position.y + position.y,
+            self.spawn_position.z + position.z
+        )
+        self.spawn_position = new_pos
+
+    def add_spawn_rotation(self, rotation: Quaternion):
+        """
+        Add rotation to the vehicle spawn rotation
+        """
+        new_rotation = Quaternion(
+            self.spawn_rotation.x + rotation.x,
+            self.spawn_rotation.y + rotation.y,
+            self.spawn_rotation.z + rotation.z,
+            self.spawn_rotation.w + rotation.w
+        )
+        self.spawn_rotation = new_rotation
+
+    def add_spawn_rotation_euler(self, rotation: Vector):
+        """
+        Add rotation to the vehicle spawn rotation
+        """
+        new_rotation = Vector(
+            self.spawn_rotation_euler.x + rotation.x,
+            self.spawn_rotation_euler.y + rotation.y,
+            self.spawn_rotation_euler.z + rotation.z
+        )
+        self.spawn_rotation_euler = new_rotation
+
+    def __repr__(self) -> str:
+        return f"Vehicle(id={self.id}, model={self.model})"
+
     def __del__(self):
         self.delete()
 
@@ -1198,6 +1374,10 @@ class Pickup:
     def id(self) -> int:
         return self._id
 
+    @property
+    def is_alive(self) -> bool:
+        return funcs.check_entity_exists(vcmpEntityPool.vcmpEntityPoolPickup, self._id)
+
     @property
     def world(self):
         return funcs.get_pickup_world(self._id)
@@ -1238,7 +1418,9 @@ class Pickup:
 
     @property
     def position(self):
-        return Vector(**funcs.get_pickup_position(self._id))
+        return Vector(
+            **funcs.get_pickup_position(self._id)
+        )
     
     @position.setter
     def position(self, position: Vector):
@@ -1266,6 +1448,24 @@ class Pickup:
         """
         funcs.refresh_pickup(self._id)
         
+    def is_streamed_for_player(self, player: int | Player) -> bool:
+        """
+        Check if the pickup is streamed for the player
+        """
+        id = player if isinstance(player, int) else player.id
+        return funcs.is_pickup_streamed_for_player(self._id, id)
+    
+    def add_position(self, position: Vector):
+        """
+        Add position to the pickup position
+        """
+        new_pos = Vector(
+            self.position.x + position.x,
+            self.position.y + position.y,
+            self.position.z + position.z
+        )
+        self.position = new_pos
+
     def delete(self):
         """
         Delete the pickup
@@ -1295,6 +1495,10 @@ class CheckPoint:
     def id(self) -> int:
         return self._id
 
+    @property
+    def is_alive(self) -> bool:
+        return funcs.check_entity_exists(vcmpEntityPool.vcmpEntityPoolCheckPoint, self._id)
+
     @property
     def sphere(self):
         return funcs.is_check_point_sphere(self._id)
@@ -1309,7 +1513,9 @@ class CheckPoint:
 
     @property
     def position(self):
-        return Vector(**funcs.get_check_point_position(self._id))
+        return Vector(
+            **funcs.get_check_point_position(self._id)
+        )
 
     @position.setter
     def position(self, position: Vector):
@@ -1348,6 +1554,17 @@ class CheckPoint:
         id = player if isinstance(player, int) else player.id
         return funcs.is_check_point_streamed_for_player(self._id, id)
 
+    def add_position(self, position: Vector):
+        """
+        Add position to the pickup position
+        """
+        new_pos = Vector(
+            self.position.x + position.x,
+            self.position.y + position.y,
+            self.position.z + position.z
+        )
+        self.position = new_pos
+
     def __del__(self):
         self.delete()
 
@@ -1369,6 +1586,10 @@ class Object:
     def id(self) -> int:
         return self._id
     
+    @property
+    def is_alive(self) -> bool:
+        return funcs.check_entity_exists(vcmpEntityPool.vcmpEntityPoolObject, self._id)
+
     @property
     def model(self):
         return funcs.get_object_model(self._id)
@@ -1387,7 +1608,9 @@ class Object:
 
     @property
     def position(self):
-        return Vector(**funcs.get_object_position(self._id))
+        return Vector(
+            **funcs.get_object_position(self._id)
+        )
 
     @position.setter
     def position(self, position: Vector):
@@ -1422,6 +1645,17 @@ class Object:
         id = player if isinstance(player, int) else player.id
         return funcs.is_object_streamed_for_player(self._id, id)
     
+    def add_position(self, position: Vector):
+        """
+        Add position to the pickup position
+        """
+        new_pos = Vector(
+            self.position.x + position.x,
+            self.position.y + position.y,
+            self.position.z + position.z
+        )
+        self.position = new_pos
+
     def set_alpha(self, alpha: int, duration: int = 0):
         funcs.set_object_alpha(self._id, alpha, duration)
 
@@ -1437,8 +1671,7 @@ class Object:
         """
         funcs.move_object_by(self._id, position.x, position.y, position.z, duration)
 
-
-    def rotate_to(self, rotation: Quaterion, duration: int = 0):
+    def rotate_to(self, rotation: Quaternion, duration: int = 0):
         """
         Rotate the object to the given rotation
         """
@@ -1450,7 +1683,7 @@ class Object:
         """
         funcs.rotate_object_to_euler(self._id, rotation.x, rotation.y, rotation.z, duration)
 
-    def rotate_by(self, rotation: Quaterion, duration: int = 0):
+    def rotate_by(self, rotation: Quaternion, duration: int = 0):
         """
         Rotate the object by the given rotation
         """
@@ -1462,7 +1695,6 @@ class Object:
         """
         funcs.rotate_object_by_euler(self._id, rotation.x, rotation.y, rotation.z, duration)
         
-
     def __del__(self):
         self.delete()
 
@@ -1483,6 +1715,10 @@ class Marker:
     def id(self) -> int:
         return self._id
 
+    @property
+    def is_alive(self) -> bool:
+        return funcs.check_entity_exists(vcmpEntityPool.vcmpEntityPoolBlip, self._id)
+
     @property
     def world(self):
         return funcs.get_coord_blip_info(self._id)[0]
